/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 06/05/2008
   derniere modification : 06/05/2008 */

#ifndef SCEOCTREE_H
#define SCEOCTREE_H

#include <SCE/utils/SCEVector.h>
#include <SCE/interface/SCEBoundingBox.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \ingroup octree
 * @{
 */

/** \copydoc sce_soctree */
typedef struct sce_soctree SCE_SOctree;

/**
 * \brief Type for depth-control function of SCE_Octree_RecursiveMake()
 * \param tree the current octree
 * \param param user-defined data
 * \returns SCE_TRUE to stop generation of octree's chlidren or SCE_FALSE to
 *          continue recursive generation
 * 
 * This type is used to control when stop octree's children generation when
 * doing a recursinve children generation.
 * 
 * \see SCE_Octree_RecursiveMake()
 */
/* prototype de la fonction d'analyse de la limite de creation d'un arbre */
typedef int (*SCE_FOctreeLimitFunc)(SCE_SOctree* tree, void* param);

/**
 * \brief Octree definition structure
 * \warning You shouldn't directly access to any of the members of this
 *          structure.
 */
struct sce_soctree
{
    SCE_SOctree *child[8];  /**< Array of octree's children */
    int have_parent;        /**< Is this octree having a parent */
    int have_children;      /**< Is this octree having children */
    SCE_SBoundingBox box;   /**< Octree's bounding box */
};

/** @} */

/* cree un octree */
SCE_SOctree* SCE_Octree_Create (void);
/* supprime un octree */
void SCE_Octree_Delete (SCE_SOctree*);
/* de facon de recursive */
void SCE_Octree_DeleteRecursive (SCE_SOctree*);

/* fonctions agissant directement sur la bounding box */
void SCE_Octree_SetCenter (SCE_SOctree*, float, float, float);
void SCE_Octree_SetCenterv (SCE_SOctree*, SCE_TVector3);
void SCE_Octree_GetCenterv (SCE_SOctree*, SCE_TVector3);
void SCE_Octree_SetSize (SCE_SOctree*, float, float, float);
float SCE_Octree_GetWidth (SCE_SOctree*);
float SCE_Octree_GetHeight (SCE_SOctree*);
float SCE_Octree_GetDepth (SCE_SOctree*);

/* retourne tous les fils d'un noeud d'octree */
SCE_SOctree** SCE_Octree_GetChildren (SCE_SOctree*);

/* construit les 8 fils d'un noeud d'octree */
int SCE_Octree_MakeChildren (SCE_SOctree*);
/* construit un arbre de facon recursive */
int SCE_Octree_RecursiveMake (SCE_SOctree*, SCE_FOctreeLimitFunc, void*);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
